/*
 *  cbrt() - cube root
 *
 *  This file is a staight-forward translation of a VHDL package.  The
 *  original VHDL package is the file mathpack.vhd in the directory:
 *
 *  http://tech-www.informatik.uni-hamburg.de/vhdl/packages/mathpack
 *
 *  The mathpack.vhd file has the following license:
 *
 *  "This source file may be used and distributed without restriction.
 *   No declarations or definitions shall be added to this package.
 *   This package cannot be sold or distributed for profit."
 *
 *  The translation into C was performed by Joel Sherrill <joel@OARcorp.com>
 *  and the original license applies to the C version.
 *
 *  cbrt.c,v 1.1.1.1 2000/11/13 21:58:23 jennifer Exp
 */

#ifndef DATA_TYPE
#include "double_setup.h"
#endif

/* NOTES:
 *   returns cube root of X
 *   Computes square root using the Newton-Raphson approximation:
 *         F(n+1) = (1/3)*[2*F(n) + x/F(n)**2];
 */

DATA_TYPE NAME(cbrt)(DATA_TYPE _x)
{
  SUBDATA_TYPE       oldval;
  SUBDATA_TYPE       newval;
  SUBDATA_TYPE       x            = (SUBDATA_TYPE) _x;
  const SUBDATA_TYPE inival       = 1.5;
  const SUBDATA_TYPE eps          = 0.000001;
  const SUBDATA_TYPE relative_err = eps*SUBNAME(fabs)(x);
  SUBDATA_TYPE       xlocal       = x;
  int                negative     = (x < 0.0);

  /* compute root for special cases */
  if ( x == 0.0 )
    return 0.0;
  else if ( x == 1.0 )
    return 1.0;
  else if ( x == -1.0 )
    return -1.0;

  /* compute root for general cases */
  if ( negative )
    xlocal = -x;

  oldval = inival;
  newval = (xlocal/(oldval*oldval) + 2.0*oldval)/3.0;

  while ( SUBNAME(fabs)(newval -oldval) > relative_err ) {
    oldval = newval;
    newval =(xlocal/(oldval*oldval) + 2.0*oldval)/3.0;
  }

  if ( negative )
    newval = -newval;

  return newval;
}
