/*
 *  CORDIC Evaluation Routine
 *
 *  This file is a staight-forward translation of a VHDL package.  The
 *  original VHDL package is the file mathpack.vhd in the directory:
 *
 *  http://tech-www.informatik.uni-hamburg.de/vhdl/packages/mathpack
 *
 *  The mathpack.vhd file has the following license:
 *
 *  "This source file may be used and distributed without restriction.
 *   No declarations or definitions shall be added to this package.
 *   This package cannot be sold or distributed for profit."
 *
 *  The translation into C was performed by Joel Sherrill <joel@OARcorp.com>
 *  and the original license applies to the C version.
 *
 *  COPYRIGHT:
 *    Copyright (c) 2000, On-Line Applications Research Corporation (OAR)
 *
 *  cordic.c,v 1.1.1.1 2000/11/13 21:58:23 jennifer Exp
 */


#ifndef DATA_TYPE
#include "double_setup.h"
#endif
#include "cordic_.h"

#define MAXIMUM_ITERATIONS 27
#define ROTATION 0
#define VECTORING 1

const double epsilon[] = {
  7.8539816339744827e-01,
  4.6364760900080606e-01,
  2.4497866312686413e-01,
  1.2435499454676144e-01,
  6.2418809995957351e-02,
  3.1239833430268277e-02,
  1.5623728620476830e-02,
  7.8123410601011116e-03,
  3.9062301319669717e-03,
  1.9531225164788189e-03,
  9.7656218955931937e-04,
  4.8828121119489829e-04,
  2.4414062014936175e-04,
  1.2207031189367021e-04,
  6.1035156174208768e-05,
  3.0517578115526093e-05,
  1.5258789061315760e-05,
  7.6293945311019699e-06,
  3.8146972656064960e-06,
  1.9073486328101870e-06,
  9.5367431640596080e-07,
  4.7683715820308876e-07,
  2.3841857910155801e-07,
  1.1920928955078067e-07,
  5.9604644775390553e-08,
  2.9802322387695303e-08,
  1.4901161193847654e-08,
  7.4505805969238281e-09
};

const double two_at_minus[32] = {
           1.0 / 1.0,  /* 1 / (2 ** 0) */ 
           1.0 / 2.0,  /* 1 / (2 ** 1) */ 
           1.0 / 4.0,  /* 1 / (2 ** 2) */ 
           1.0 / 8.0,  /* 1 / (2 ** 3) */ 
          1.0 / 16.0,  /* 1 / (2 ** 4) */ 
          1.0 / 32.0,  /* 1 / (2 ** 5) */ 
          1.0 / 64.0,  /* 1 / (2 ** 6) */ 
         1.0 / 128.0,  /* 1 / (2 ** 7) */ 
         1.0 / 256.0,  /* 1 / (2 ** 8) */ 
         1.0 / 512.0,  /* 1 / (2 ** 9) */ 
        1.0 / 1024.0,  /* 1 / (2 ** 10) */ 
        1.0 / 2048.0,  /* 1 / (2 ** 11) */ 
        1.0 / 4096.0,  /* 1 / (2 ** 12) */ 
        1.0 / 8192.0,  /* 1 / (2 ** 13) */ 
       1.0 / 16384.0,  /* 1 / (2 ** 14) */ 
       1.0 / 32768.0,  /* 1 / (2 ** 15) */ 
       1.0 / 65536.0,  /* 1 / (2 ** 16) */ 
      1.0 / 131072.0,  /* 1 / (2 ** 17) */ 
      1.0 / 262144.0,  /* 1 / (2 ** 18) */ 
      1.0 / 524288.0,  /* 1 / (2 ** 19) */ 
     1.0 / 1048576.0,  /* 1 / (2 ** 20) */ 
     1.0 / 2097152.0,  /* 1 / (2 ** 21) */ 
     1.0 / 4194304.0,  /* 1 / (2 ** 22) */ 
     1.0 / 8388608.0,  /* 1 / (2 ** 23) */ 
    1.0 / 16777216.0,  /* 1 / (2 ** 24) */ 
    1.0 / 33554432.0,  /* 1 / (2 ** 25) */ 
    1.0 / 67108864.0,  /* 1 / (2 ** 26) */ 
   1.0 / 134217728.0,  /* 1 / (2 ** 27) */ 
   1.0 / 268435456.0,  /* 1 / (2 ** 28) */ 
   1.0 / 536870912.0,  /* 1 / (2 ** 29) */ 
  1.0 / 1073741824.0,  /* 1 / (2 ** 30) */ 
};

void NAME(CORDIC)(
  DATA_TYPE x0,
  DATA_TYPE y0, 
  DATA_TYPE z0,
  int    n,                  /* precision factor */
  int    cordic_mode,        /* rotation (z -> 0) or vectoring (y -> 0) */
  DATA_TYPE *xout,
  DATA_TYPE *yout,
  DATA_TYPE *zout
)
{
  DATA_TYPE x = x0;
  DATA_TYPE y = y0;
  DATA_TYPE z = z0;
  DATA_TYPE x_temp; 
  int k;

  if ( n > MAXIMUM_ITERATIONS )
    return;

  if ( cordic_mode == ROTATION ) {
     for ( k=0; k<=n; k++ ) {
        x_temp = x;
        if ( z >= 0.0 ) {
           x = x - y * two_at_minus[k];
           y = y + x_temp * two_at_minus[k]; 
           z = z - epsilon[k];
        } else {
           x = x + y * two_at_minus[k];
           y = y - x_temp * two_at_minus[k];
           z = z + epsilon[k];
        }
     }
  } else  {
     for ( k=0; k<=n; k++ ) {
        x_temp = x;
        if ( y < 0.0 ) {
           x = x - y * two_at_minus[k];
           y = y + x_temp * two_at_minus[k]; 
           z = z - epsilon[k];
        } else {
           x = x + y * two_at_minus[k];
           y = y - x_temp * two_at_minus[k];
           z = z + epsilon[k];
        }
     }
  }

  if (xout) *xout = x;
  if (yout) *yout = y;
  if (zout) *zout = z;
}










