/*
 *  exp() - exponential
 *
 *  This file is a staight-forward translation of a VHDL package.  The
 *  original VHDL package is the file mathpack.vhd in the directory:
 *
 *  http://tech-www.informatik.uni-hamburg.de/vhdl/packages/mathpack
 *
 *  The mathpack.vhd file has the following license:
 *
 *  "This source file may be used and distributed without restriction.
 *   No declarations or definitions shall be added to this package.
 *   This package cannot be sold or distributed for profit."
 *
 *  The translation into C was performed by Joel Sherrill <joel@OARcorp.com>
 *  and the original license applies to the C version.
 *
 *  exp.c,v 1.1.1.1 2000/11/13 21:58:23 jennifer Exp
 */

#ifndef DATA_TYPE
#include "double_setup.h"
#endif

DATA_TYPE NAME(exp)(DATA_TYPE _x)
{
  /* returns e**X; where e = MATH_E 
   *
   * This function computes the exponential using the following series:
   *    exp(x) = 1 + x + x**2/2! + x**3/3! + ... ; x > 0
   */

  SUBDATA_TYPE x = (SUBDATA_TYPE) _x;
  SUBDATA_TYPE xlocal;
  SUBDATA_TYPE oldval;                 /* following variables are */
  SUBDATA_TYPE numerator;              /* used for series evaluation */
  int          count;
  SUBDATA_TYPE denominator;
  SUBDATA_TYPE result;

  xlocal = SUBNAME(fabs)(x);  /* use positive value */

  /* handle special cases */
  if ( x == 0.0 ) 
    return 1.0;

  if ( x == 1.0 )
    return M_E;

  /* compute value for general cases */
  oldval = 1.0;
  numerator = xlocal;
  count = 1;
  denominator = 1.0;
  result= oldval + numerator/denominator;

  while ( SUBNAME(fabs)(result - oldval) > EXP_EPSILON ) {
    oldval = result;
    numerator *= xlocal;
    count++;
    denominator *= (double) count;
    result = oldval + numerator/denominator;
  }

  if ( x < 0.0 )
    result = 1.0/result;

  return result;
}
