/*
 *  log() - natural logarithms
 *
 *  This file is a staight-forward translation of a VHDL package.  The
 *  original VHDL package is the file mathpack.vhd in the directory:
 *
 *  http://tech-www.informatik.uni-hamburg.de/vhdl/packages/mathpack
 *
 *  The mathpack.vhd file has the following license:
 *
 *  "This source file may be used and distributed without restriction.
 *   No declarations or definitions shall be added to this package.
 *   This package cannot be sold or distributed for profit."
 *
 *  The translation into C was performed by Joel Sherrill <joel@OARcorp.com>
 *  and the original license applies to the C version.
 *
 *  log.c,v 1.1.1.1 2000/11/13 21:58:23 jennifer Exp
 */


#ifndef DATA_TYPE
#include "double_setup.h"
#endif

DATA_TYPE NAME(log)(DATA_TYPE _x)
{
  /* returns natural logarithm of X; X > 0
   *
   * This function computes the exponential using the following series:
   *    log(x) = 2[ (x-1)/(x+1) + (((x-1)/(x+1))**3)/3.0 + ...] ; x > 0
   */

  SUBDATA_TYPE x = (SUBDATA_TYPE) _x;
  SUBDATA_TYPE xlocal;                 /* following variables are */
  SUBDATA_TYPE oldval;                 /* used to evaluate the series */
  SUBDATA_TYPE xlocalsqr;
  SUBDATA_TYPE factor;
  int          count;
  SUBDATA_TYPE result;

  /* check validity of argument */
  if ( x <= 0.0 ) {
    return NAME(nan)();

  }

  /* compute value for special cases */
  if ( x == 1.0 )
    return 0.0;
  if ( x == M_E )
    return 1.0;

  /* compute value for general cases */
  xlocal = (x - 1.0)/(x + 1.0);
  oldval = xlocal;
  xlocalsqr = xlocal*xlocal;
  factor = xlocal*xlocalsqr;
  count = 3;
  result = oldval + (factor/(SUBDATA_TYPE)count);

  while ( SUBNAME(fabs)(result - oldval) > LOG_EPSILON ) {
    oldval = result;
    count += 2;
    factor *= xlocalsqr;
    result = oldval + factor/(SUBDATA_TYPE)count;
  }

  return (DATA_TYPE)(result * 2.0);
}




