/*
 *  pow() - XXX
 *
 *  Formula taken from VHDL, error checking from newlib
 *
 *  COPYRIGHT:
 *    Copyright (c) 2000, On-Line Applications Research Corporation (OAR)
 *
 *  pow.c,v 1.1.1.1 2000/11/13 21:58:23 jennifer Exp
 */


#ifndef DATA_TYPE
#include "double_setup.h"
#endif

/****************************************************************
 * FUNCTION:    pow
 * DESCRIPTION: Calculates x^y (x raised to the power y)
 * INPUT:
 *  PARAMETERS: x - double, y - double
 *  GLOBALS:    none
 * OUTPUT:
 *  RESULT:     double - x^y
 *  GLOBALS:    none
 * CALLS:       none
 * COMMENTS: 
 *   Special cases:                                from newlib
 *      1.  (anything) ** 0  is 1
 *      2.  (anything) ** 1  is itself
 *      3.  (anything) ** NAN is NAN
 *      4.  NAN ** (anything except 0) is NAN
 *      5.  +-(|x| > 1) **  +INF is +INF
 *      6.  +-(|x| > 1) **  -INF is +0
 *      7.  +-(|x| < 1) **  +INF is +0
 *      8.  +-(|x| < 1) **  -INF is +INF
 *      9.  +-1         ** +-INF is NAN
 *      10. +0 ** (+anything except 0, NAN)               is +0
 *      11. -0 ** (+anything except 0, NAN, odd integer)  is +0
 *      12. +0 ** (-anything except 0, NAN)               is +INF
 *      13. -0 ** (-anything except 0, NAN, odd integer)  is +INF
 *      14. -0 ** (odd integer) = -( +0 ** (odd integer) )
 *      15. +INF ** (+anything except 0,NAN) is +INF
 *      16. +INF ** (-anything except 0,NAN) is +0
 *      17. -INF ** (anything)  = -0 ** (-anything)
 *      18. (-anything) ** (integer) is (-1)**(integer)*(+anything**integer)
 *      19. (-anything except 0 and inf) ** (non-integer) is NAN
 ****************************************************************/

DATA_TYPE NAME( pow )(DATA_TYPE _x, DATA_TYPE _y)
{
  SUBDATA_TYPE x = (SUBDATA_TYPE) _x;
  SUBDATA_TYPE y = (SUBDATA_TYPE) _y;
  SUBDATA_TYPE iptr;
  SUBDATA_TYPE sign = 1;
  int          i;

  if ( y == 0 )    /* 1 */
    return 1.0;

  if ( y == 1 )    /* 2 */
    return x;

  if ( x == 0.0 ) {
    if ( y > 0)    /* 10 */
      return 0;
    else
      return Inf;  /* 12 */
  }

  if ( x == -0.0 ) {
    iptr = SUBNAME(floor)( y );
    if ( iptr == y ) {
      i = iptr;
      if ( ( ( i >> 1 ) << 1) != i ){  /* If Odd */
        return NInf;   /* XXX - 14 */
      }
    }
    x *= -1;
    if ( y > 0.0 )    /* 11 */
      return 0.0;
    else
      return Inf;  /* 13 */
  }

  if ( x < 0.0 ) {
    iptr = SUBNAME(floor)( y );
    if ( iptr == y ) {
      x = x * -1;  /* 18 */
      i = iptr;
      if ( ( ( i >> 1 ) << 1) != i ) /* If Odd */
        sign = -1; 
    } else
      return NaN;  /* 19 */
  }

  return (DATA_TYPE)( sign * SUBNAME(exp)(y * SUBNAME(log)(x) ));
}


