/*
 *  sqrt() - square root
 *
 *  This file is a staight-forward translation of a VHDL package.  The
 *  original VHDL package is the file mathpack.vhd in the directory:
 *
 *  http://tech-www.informatik.uni-hamburg.de/vhdl/packages/mathpack
 *
 *  The mathpack.vhd file has the following license:
 *
 *  "This source file may be used and distributed without restriction.
 *   No declarations or definitions shall be added to this package.
 *   This package cannot be sold or distributed for profit."
 *
 *  The translation into C was performed by Joel Sherrill <joel@OARcorp.com>
 *  and the original license applies to the C version.
 *
 *  sqrt.c,v 1.1.1.1 2000/11/13 21:58:23 jennifer Exp
 */


#ifndef DATA_TYPE
#include "double_setup.h"
#endif

DATA_TYPE NAME(sqrt)(DATA_TYPE _x)
{
  /*
   *  returns square root of X;  X >= 0
   *
   *  Computes square root using the Newton-Raphson approximation:
   *  F(n+1) = 0.5*[F(n) + x/F(n)];
   */
  
  SUBDATA_TYPE x = (SUBDATA_TYPE) _x;
  const SUBDATA_TYPE  inival = 1.5;
  const SUBDATA_TYPE relative_err = SQRT_EPSILON * x;
  SUBDATA_TYPE oldval;
  SUBDATA_TYPE result;

  /* check validity of argument */
  if ( x < 0.0 ) {
    return NAME(nan)();
  }

  /* get the square root for special cases */
  if ( x == 0.0 )
    return 0.0;

  if ( x == 1.0 )
    return 1.0; /* return exact value */

  /* get the square root for general cases */
  oldval = inival;
  result = (x/oldval + oldval)/2.0;

  while ( SUBNAME(fabs)(result - oldval) > relative_err ) {
    oldval = result;
    result = (x/oldval + oldval)/2.0;
  }

  return (DATA_TYPE)result;
}
