**************************************************************************
*   PROGRAM: EXP                                                         *
*                                                                        *
*   WRITTEN BY: GARY A. SITTON                                           *
*               GAS LIGHT SOFTWARE                                       *
*               HOUSTON, TEXAS                                           *
*               MARCH 1989.                                              *
*                                                                        *
*   EXPONENTIAL FUNCTION: R0 <= EXP(R0).                                 *
*                                                                        *
*   APPROXIMATE ACCURACY: 7 DECIMAL DIGITS.                              *
*   INPUT RESTRICTIONS: |R0| <= 88.0.                                    *
*   REGISTERS FOR INPUT: R0.                                             *
*   REGISTERS USED AND RESTORED: DP AND SP.                              *
*   REGISTERS ALTERED: R0-4.                                             *
*   REGISTERS FOR OUTPUT: R0.                                            *
*   ROUTINES NEEDED: FPINV.                                              *
*   EXECUTION CYCLES (MIN, MAX): 44 (R0 <= 0), 70.                       *
*	                                                                 *              
*  COPYRIGHT:                                                            *
*    Copyright (c) 2000, On-Line Applications Research Corporation (OAR) *
*	                                                                 *              
*  _exp.S,v 1.1.1.1 2000/11/13 21:58:23 jennifer Exp                                                                  *
**************************************************************************

;       EXTERNAL PROGRAM NAMES

        .GLOBL  __exp
        .GLOBL  FPINV

;       INTERNAL CONSTANTS

        .DATA

;       SCALING COEFF. FOR 2**-X

ENRM    .FLOAT  1.442695041     ; 1/LN(2)

;       POLYNOMIAL COEFFS. FOR 2**-X, 0 <= X < 1.

        .FLOAT  1.0000000000    ; C0
        .FLOAT  -0.693147180    ; C1
        .FLOAT  0.240226469     ; C2
        .FLOAT  -0.055503654    ; C3
        .FLOAT  0.009615978     ; C4
        .FLOAT  -0.001328240    ; C5
        .FLOAT  0.000147491     ; C6
C7      .FLOAT  -0.000010863    ; C7

AC7     .WORD   C7

        .TEXT

;       START OF EXP PROGRAM

__exp:

;       SCALE VARIABLE X

        PUSH    DP              ; SAVE DP
        LDP     @AC7            ; LOAD DATA PAGE POINTER
	LDF	R2,R0		; XXX- COPY R2 TO R0

        RND     R0              ; ROUND X
        NEGF    R0,R2           ; R2 <= -X
        LDF     R0,R1           ; R1 <= X
        LDFN    R2,R1           ; IF X < 0 THEN R1 <= |X|
        MPYF    @ENRM,R1        ; R1 <= X = |X|/LN(2)
        FIX     R1,R3           ; R3 <= I = INTEGER OF X
        FLOAT   R3,R0           ; R0 <= FLT. PT. I
        SUBF    R0,R1           ; R1 <= FRACTION OF |X|, 0 <= X < 1
        NEGI    R3              ; R3 <= -I
        LSH     24,R3           ; MOVE -I TO EXP.
        PUSH    R3              ; SAVE AS INT.
        POPF    R3              ; R3 <= FLT. PT. 2**-I
        LDI     @AC7,AR0        ; AR0 -> COEFF. TABLE
        POP     DP              ; UNSAVE DP

;       EVALUATE TRUNCATED SERIES

        RND     R1              ; ROUND BEFORE *
        MPYF    *AR0--,R1,R0    ; R0 <= X*C7
        ADDF    *AR0--,R0       ; R0 <= C6 + R0

        MPYF    R1,R0           ; R0 <= X*(C6 + R0)
        ADDF    *AR0--,R0       ; R0 <= C5 + R0

        MPYF    R1,R0           ; R0 <= X*(C5 + R0)
        ADDF    *AR0--,R0       ; R0 <= C4 + R0

        MPYF    R1,R0           ; R0 <= X*(C4 + R0)
        ADDF    *AR0--,R0       ; R0 <= C3 + R0

        RND     R0              ; ROUND BEFORE *
        MPYF    R1,R0           ; R0 <= X*(C3 + R0)
        ADDF    *AR0--,R0       ; R0 <= C2 + R0

        RND     R0              ; ROUND BEFORE *
        MPYF    R1,R0           ; R0 <= X*(C2 + R0)
        ADDF    *AR0--,R0       ; R0 <= C1 + R0

        RND     R0              ; ROUND BEFORE *
        MPYF    R1,R0           ; R0 <= X*(C1 + R0)

;       TEST FOR X < 0 AND RETURN

        LDF     R2,R2           ; TEST ORIGINAL -X
        BND     FPINV           ; IF -X < 0 THEN R0 <= 1/X, (DELAYED)
        ADDF    *AR0,R0         ; R0 <= 2**-X = C0 + R0
        RND     R0              ; ROUND BEFORE *
        MPYF    R3,R0           ; R0 <= 2**-(I + X)

        RETS                    ; RETURN (IF NO FPINV BRANCH)

