 *********************************************************
*   PROGRAM: FPINV                                      *
*                                                       *
*   WRITTEN BY: GARY A. SITTON                          *
*               GAS LIGHT SOFTWARE                      *
*               HOUSTON, TEXAS                          *
*               MARCH 1989.                             *
*                                                       *
*   FLOATING POINT INVERSE: R0 <= 1/R0                  *
*                                                       *
*   APPROXIMATE ACCURACY: 8 DECIMAL DIGITS.             *
*   INPUT RESTRICTIONS: R0 != 0.0.                      *
*   REGISTERS FOR INPUT: R0.                            *
*   REGISTERS USED AND RESTORED: DP AND SP.             *
*   REGISTERS ALTERED: R0-2 AND R4.                     *
*   REGISTERS FOR OUTPUT: R0.                           *
*   ROUTINES NEEDED: NONE.                              *
*   EXECUTION CYCLES (MIN, MAX): 33 , 33.               *
*********************************************************

;       EXTERNAL PROGRAM NAMES

        .GLOBL  FPINV

;       INTERNAL CONSTANTS

        .DATA

ONE     .FLOAT  1.0
TWO     .FLOAT  2.0

MSK     .WORD   0FF7FFFFFH

        .TEXT

;       START OF FPINV PROGRAM

FPINV:

        LDF     R0,R0           ; TEST F
        RETSZ                   ; RETURN NOW IF F = 0

;       GET APPROXIMATION TO 1/F.  FOR F = (1+M) * 2**E
;       AND 0 <= M < 1, USE: X[0] = (1-M/2) * 2**-E

        PUSH    DP              ; SAVE DATA PAGE POINTER
        LDP     @MSK            ; LOAD DATA PAGE POINTER
        PUSHF   R0              ; SAVE AS FLT. PT. F = (1+M) * 2**E
        POP     R1              ; FETCH BACK AS INTEGER
        XOR     @MSK,R1         ; COMPLEMENT E & M BUT NOT SIGN BIT
        PUSH    R1              ; SAVE AS INTEGER, AND BY MAGIC...
        POPF    R1              ; R1 <= X[0] = (1-M/2) * 2**-E.
        POP     DP              ; UNSAVE DP

;       NEWTON ITERATION FOR: Y(X) = X - 1/F = 0 ...

        MPYF    R1,R0,R4        ; R4 <= F * X[0]
        SUBRF   TWO,R4          ; R4 <= 2 - F * X[0]
        MPYF    R4,R1           ; R1 <= X[1] = X[0] * (2 - F * X[0])

        MPYF    R1,R0,R4        ; R4 <= F * X[1]
        SUBRF   TWO,R4          ; R4 <= 2 - F * X[1]
        MPYF    R4,R1           ; R1 <= X[2] = X[1] * (2 - F * X[1])

        MPYF    R1,R0,R4        ; R4 <= F * X[2]
        SUBRF   TWO,R4          ; R4 <= 2 - F * X[2]
        MPYF    R4,R1           ; R1 <= X[3] = X[2] * (2 - F * X[2])

;       FOR THE LAST ITERATION: X[4] = (X[3] * (1 - (F * X[3]))) + X[3]

        RND     R0,R4           ; ROUND F BEFORE LAST MULTIPLY
        RND     R1,R0           ; ROUND X[3] BEFORE MULTIPLIES
        MPYF    R0,R4           ; R4 <= F * X[3] = 1 + EPS

;       FINISH ITERATION AND RETURN

        POP     R2              ; R2 <= RETURN ADDRESS
        BUD     R2              ; RETURN (DELAYED)
        SUBRF   ONE,R4          ; R4 <= 1 - F * X[3] = EPS
        MPYF    R0,R4           ; R4 <= X[3] * EPS
        ADDF    R4,R1,R0        ; R0 <= X[4] = (X[3]*(1 - (F*X[3]))) + X[3]

