**************************************************************************
*   PROGRAM: LN                                                          *
*                                                                        *
*   WRITTEN BY: GARY A. SITTON                                           *
*               GAS LIGHT SOFTWARE                                       *
*               HOUSTON, TEXAS                                           *
*               MARCH 1989.                                              *
*                                                                        *
*   LOGARITHM FUNCTION BASE E: R0 <= LN(R0).                             *
*                                                                        *
*   APPROXIMATE ACCURACY: 7 DECIMAL DIGITS.                              *
*   INPUT RESTRICTIONS: R0 > 0.0.                                        *
*   REGISTERS FOR INPUT: R2                                              *
*   REGISTERS USED AND RESTORED: DP AND SP.                              *
*   REGISTERS ALTERED: AR0 AND R0-3.                                     *
*   REGISTERS FOR OUTPUT: R0.                                            *
*   ROUTINES NEEDED: NONE.                                               *
*   EXECUTION CYCLES (MIN, MAX): 43 , 43.                                *
*	                                                                 *              
*  COPYRIGHT:                                                            *
*    Copyright (c) 2000, On-Line Applications Research Corporation (OAR) *
*	                                                                 *              
*  _log.S,v 1.1.1.1 2000/11/13 21:58:23 jennifer Exp                                                                  *
**************************************************************************

;       EXTERNAL PROGRAM NAMES

        .GLOBL  __log

;       INTERNAL CONSTANTS

        .DATA

;       SCALING COEFFS. FOR LN(1+X)

LNRM    .FLOAT  0.6931471806    ; LN(2)
C0      .FLOAT  1.0000000000    ; C0 (1.0)

;       POLYNOMIAL COEFFS. FOR LN(1+X), 0 <= X < 1.

        .FLOAT  0.9999964239    ; TOP OF C1
        .FLOAT  -0.4998741238   ; TOP OF C2
        .FLOAT  0.3317990258    ; TOP OF C3
        .FLOAT  -0.2407338084   ; TOP OF C4
        .FLOAT  0.1676540711    ; TOP OF C5
        .FLOAT  -0.0953293897   ; TOP OF C6
        .FLOAT  0.0360884937    ; TOP OF C7
C8      .FLOAT  -0.0064535442   ; TOP OF C8
NINF	.FLOAT	5.8774718E-39

AC8     .WORD   C8

        .TEXT

;       START OF LN PROGRAM

__log:
        LDF     R2,R2           ; TEST X
        RETSLE                  ; RETURN NOW IF X <= 0

;       SCALE VARIABLE X

        PUSH    DP              ; SAVE DP
        LDP     @AC8            ; LOAD DATA PAGE POINTER
	LDF	R2,R0		; XXX- COPY R2 TO R0

        PUSHF   R0              ; SAVE AS FLT. PT.
        POP     R3              ; R3 <= INTEGER FORMAT
        ASH     -24,R3          ; R3 <= E = SIGNED EXP.
        FLOAT   R3,R1           ; R1 <= FLT. PT. E VALUE
        LDF     @C0,R2          ; R2 <= 1.0
        LDE     R2,R0           ; EXP. R0 <= 0 (1 <= X < 2)
        SUBRF   R0,R2           ; R2 <= X - 1 (0 <= X < 1)
        LDF     @LNRM,R0        ; R0 <= LN(2)
        MPYF    R1,R0           ; R0 <= E*LN(2)
        LDF     R0,R3           ; R3 <= E*LN(2)
        LDI     @AC8,AR0        ; AR0 -> COEFF. TABLE
        POP     DP              ; UNSAVE DP

;       EVALUATE TRUNCATED SERIES

        RND     R2,R1           ; R1 <= RND X
        MPYF    *AR0--,R1,R0    ; R0 <= X*C8
        ADDF    *AR0--,R0       ; R0 <= C7 + R0

        MPYF    R1,R0           ; R0 <= X*(C7 + R0)
        ADDF    *AR0--,R0       ; R0 <= C6 + R0

        MPYF    R1,R0           ; R0 <= X*(C6 + R0)
        ADDF    *AR0--,R0       ; R0 <= C5 + R0

        MPYF    R1,R0           ; R0 <= X*(C5 + R0)
        ADDF    *AR0--,R0       ; R0 <= C4 + R0

        MPYF    R1,R0           ; R0 <= X*(C4 + R0)
        ADDF    *AR0--,R0       ; R0 <= C3 + R0

        RND     R0              ; ROUND BEFORE *
        MPYF    R1,R0           ; R0 <= X*(C3 + R0)
        ADDF    *AR0--,R0       ; R0 <= C2 + R0

        RND     R0              ; ROUND BEFORE *
        MPYF    R1,R0           ; R0 <= X*(C2 + R0)
        ADDF    *AR0--,R0       ; R0 <= C1 + R0

;       ADD IN SCALED EXPONENT AND RETURN

        POP     R2              ; R2 <= RETURN ADDRESS
        BUD     R2              ; RETURN (DELAYED)
        RND     R0              ; ROUND BEFORE *
        MPYF    R1,R0           ; R0 <= X*(C1 + R0)
        ADDF    R3,R0           ; R0 <= LN(X) + E*LN(2)

