**************************************************************************
*   PROGRAM: SIN                                                         *
*                                                                        *
*   WRITTEN BY: GARY A. SITTON                                           *
*               GAS LIGHT SOFTWARE                                       *
*               HOUSTON, TEXAS                                           *
*               MARCH 1989.                                              *
*                                                                        *
*   SINE FUNCTION: R0 <= SIN(R0).                                        *
*                                                                        *
*   APPROXIMATE ACCURACY: 7 DECIMAL DIGITS.                              *
*   INPUT RESTRICTIONS: NONE.                                            *
*   REGISTERS FOR INPUT: R2 WAS R0 (ARGUMENT IN RADIANS).                *
*   REGISTERS USED AND RESTORED: DP AND SP.                              *
*   REGISTERS ALTERED: AR0, IR0, AND R0-4.                               *
*   REGISTERS FOR OUTPUT: R0.                                            *
*   ROUTINES NEEDED: NONE.                                               *
*   EXECUTION CYCLES (MIN, MAX): 44 , 44.                                *
*	                                                                 *              
*  COPYRIGHT:                                                            *
*    Copyright (c) 2000, On-Line Applications Research Corporation (OAR) *
*	                                                                 *              
*  _sin.S,v 1.1.1.1 2000/11/13 21:58:23 jennifer Exp                                                                  *
**************************************************************************

;       EXTERNAL PROGRAM NAMES

        .GLOBL  _sin
        .GLOBL  ECOS

;       INTERNAL CONSTANTS

        .DATA

NORM    .FLOAT  0.636619772     ; 2/PI

;       POLYNOMIAL COEFFS. FOR SIN(X*2/PI), -1 < X < 1

SHFT    .FLOAT  1.570796327     ; C1 (PI/2)
        .FLOAT  -0.6459640968   ; C3
        .FLOAT  0.07969260878   ; C5
        .FLOAT  -0.00468166687  ; C7
        .FLOAT  0.00016025884   ; C9
COF     .FLOAT  -0.000003433338 ; C11

ACOF    .WORD   COF             ; ADDRESS OF COEFFS.

CON     .FLOAT  -1.0, 0.0, 1.0, 0.0 ; MAPPING CONSTANTS

ACON    .WORD   CON             ; ADDRESS OF CONSTS.

        .TEXT

;       START OF SIN PROGRAM

_sin:
        PUSH    DP              ; SAVE DP
        LDP     @ACOF           ; LOAD DATA PAGE POINTER
	LDF	R2,R0		; XXX- COPY R2 TO R0
        RND     R0              ; ROUND X
        LDF     R0,R4           ; R4 <= X

;       COSINE ENTRY POINT

ECOS:

;       SCALE AND MAP VARIABLE X

        ABSF    R0              ; R0 <= |X|
        LDF     R0,R1           ; R1 <= RND |X|
        MPYF    @NORM,R1        ; R1 <= X*2/PI
        FIX     R1,IR0          ; IR0 <= INTEGER QUADRANT Q
        FLOAT   IR0,R2          ; R2 <= FLOATING QUADRANT Q
        SUBF    R2,R1,R0        ; R0 <= X, -1 < X < 1
        NEGF    R0,R3           ; R3 <= -X
        ADDI    1,IR0           ; IR0 <= Q + 1
        AND     3,IR0           ; IR0 <= TABLE INDEX
        TSTB    2,IR0           ; LOOK AT 2ND LSB
        LDFNZ   R3,R0           ; IF 1 THEN R0 <= -X
        LDI     @ACON,AR0       ; AR1 -> CONST. TABLE
        ADDF    *+AR0(IR0),R0   ; FINAL MAPPING, R0 <= X + C
        NEGF    R0,R3           ; R3 <= -X
        LDI     @ACOF,AR0       ; AR0 -> COEFF. TABLE
        POP     DP              ; UNSAVE DP

;       EVALUATE TRUNCATED (ODD) SERIES

        MPYF    R0,R0,R2        ; R2 <= X**2
        RND     R2              ; ROUND X**2

        MPYF    *AR0--,R2,R1    ; R1 <= X**2*C11
        ADDF    *AR0--,R1       ; R1 <= C9 + R1

        MPYF    R2,R1           ; R1 <= X**2*(C9 + R1)
        ADDF    *AR0--,R1       ; R1 <= C7 + R1

        MPYF    R2,R1           ; R1 <= X**2*(C7 + R1)
        ADDF    *AR0--,R1       ; R1 <= C5 + R1

        RND     R1              ; ROUND BEFORE *
        MPYF    R2,R1           ; R1 <= X**2*(C5 + R1)
        ADDF    *AR0--,R1       ; R1 <= C3 + R1

        RND     R1              ; ROUND BEFORE *
        MPYF    R2,R1           ; R1 <= X**2*(C3 + R1)
        ADDF    *AR0,R1         ; R1 <= C1 + R1

;       FINISH UP SERIES AND RETURN

        LDF     R4,R4           ; TEST ORIGINAL X
        LDFN    R3,R0           ; IF X < 0 THEN R0 <= -X
        POP     R2              ; R2 <= RETURN ADDRESS
        BUD     R2              ; RETURN (DELAYED)
        RND     R0              ; ROUND BEFORE *
        RND     R1              ; ROUND BEFORE *
        MPYF    R1,R0           ; R1 <= X*(C1 + R1)

*********************************************************
*   PROGRAM: COS                                        *
*                                                       *
*   WRITTEN BY: GARY A. SITTON                          *
*               GAS LIGHT SOFTWARE                      *
*               HOUSTON, TEXAS                          *
*               MARCH 1989.                             *
*                                                       *
*   COSINE FUNCTION: R0 <= COS(R0).                     *
*                                                       *
*   APPROXIMATE ACCURACY: 7 DECIMAL DIGITS.             *
*   INPUT RESTRICTIONS: NONE.                           *
*   REGISTERS FOR INPUT: R0 (ARGUMENT IN RADIANS).      *
*   REGISTERS USED AND RESTORED: DP AND SP.             *
*   REGISTERS ALTERED: AR0, IR0, AND R0-4.              *
*   REGISTERS FOR OUTPUT: R0.                           *
*   ROUTINES NEEDED: ECOS (SIN).                        *
*   EXECUTION CYCLES (MIN, MAX): 46 , 46.               *
*                                                       *
*   NOTE: USES SHFT CONSTANT FROM SIN PROGRAM!          *
*********************************************************

;       EXTERNAL PROGRAM NAMES

        .GLOBL  _cos
        .GLOBL  ECOS

        .TEXT

;       START OF COS PROGRAM

_cos:

        PUSH    DP              ; SAVE DP
        LDP     @ACOF           ; LOAD DATA PAGE POINTER
	LDF	R2,R0		; XXX- COPY R2 TO R0

        BRD     ECOS            ; R0 <= COS(X) = SIN(Xp), (DELAYED)
        RND     R0              ; ROUND X
        ADDF    @SHFT,R0        ; R0 <= Xp = X + PI/2
        LDF     R0,R4           ; R4 <= Xp

;       RETURN OCCURS FROM SIN !
