**************************************************************************
*   PROGRAM: SQRT                                                        *
*                                                                        *
*   This file was taken from Gas light source as shown                   *
*   below.  Moidifications were made to the call                         *
*   registers, name changes were made and error                          *
*   error checking is done prior to calles to these                      *
*   methods.                                                             *
*                                                                        *
*   WRITTEN BY: GARY A. SITTON                                           *
*               GAS LIGHT SOFTWARE                                       *
*               HOUSTON, TEXAS                                           *
*               MARCH 1989.                                              *
*                                                                        *
*   SQUARE ROOT FUNCTION: R0 <= SQRT(R0).                                *
*                                                                        *
*   APPROXIMATE ACCURACY: 8 DECIMAL DIGITS.                              *
*   INPUT RESTRICTIONS: R2 >= 0.0.                                       *
*   REGISTERS FOR INPUT: R0.                                             *
*   REGISTERS USED AND RESTORED: DP AND SP.                              *
*   REGISTERS ALTERED: R0-4.                                             *
*   REGISTERS FOR OUTPUT: R0.                                            *
*   ROUTINES NEEDED: NONE.                                               *
*   EXECUTION CYCLES (MIN, MAX): 49 , 49.                                *
*	                                                                 *              
*  COPYRIGHT:                                                            *
*    Copyright (c) 2000, On-Line Applications Research Corporation (OAR) *
*	                                                                 *              
*  _sqrt.S,v 1.1.1.1 2000/11/13 21:58:23 jennifer Exp                                                                  *
**************************************************************************

;       EXTERNAL PROGRAM NAMES

        .GLOBL  __sqrt

;       INTERNAL CONSTANTS

        .DATA

CNST1   .FLOAT  0.5
CNST2   .FLOAT  1.5
CNST3   .FLOAT  1.103553391     ; ADJUSTED 1.0
CNST4   .FLOAT  0.780330086     ; ADJUSTED SQRT(1/2)
NINF	.FLOAT	5.8774718E-39
SMSK    .WORD   0FF7FFFFFH

        .TEXT

;       START OF SQRT PROGRAM.

__sqrt:
	LDF      @NINF,R0       ; 
        LDF      R2,R3          ; TEST AND SAVE V
        RETSLE                  ; RETURN NOW IF V <= 0

;       GET APPROXIMATION TO 1/V.  FOR V = (1+M)*2**E
;       AND 0 <= M < 1, FOR E EVEN: X[0] = (1-M/2)*2**-E/2
;       AND FOR E ODD: X[0] = SQRT(1/2)*(1-M/2)*2**-E/2

        PUSH    DP              ; SAVE DP
        LDP     @SMSK           ; LOAD DATA PAGE POINTER
	LDF	R2,R0		; XXX- COPY R2 TO R0
        PUSHF   R0              ; SAVE V AS FLT. PT. V = (1+M)*2**E
        POP     R2              ; R2 <= V AS INTEGER
        XOR     @SMSK,R2        ; R2 <= COMPLEMENT ALL BUT SIGN
        LDI     R2,R1           ; R1 <= (1-M/2)*2**-E
        LDI     R2,R4           ; R4 <= R1
        LSH     8,R1            ; R1 <= R1 EXP. REMOVED
        ASH     -1,R2           ; R2 <= R2 WITH -E/2 EXP.
        PUSH    R2              ; SAVE R2 AS INTEGER
        POPF    R2              ; R2 <= FLT. PT.
        LDE     R2,R1           ; R1 <= (1-M/2)*2**-E/2
        LDF     @CNST3,R2       ; R2 <= 1.1... FOR ODD E
        LSH     7,R4            ; TEST LSB OF E (AS SIGN)
        LDFNN   @CNST4,R2       ; IF E EVEN R2 <= 0.78...
        MPYF    R2,R1           ; R1 <= CORRECTED ESTIMATE
        POP     DP              ; UNSAVE DP

;       GENERATE V/2 (USES MPYF).

        MPYF    CNST1,R0        ; R0 <= V/2 TRUNC.
        RND     R0              ; R0 <= RND V/2

;       NEWTON ITERATION FOR Y(X) = X - V**-2 = 0 ...

        MPYF    R1,R1,R2        ; R2 <= X[0]**2
        MPYF    R0,R2           ; R2 <= (V/2) * X[0]**2
        SUBRF   CNST2,R2        ; R2 <= 1.5 - (V/2) * X[0]**2
        MPYF    R2,R1           ; R1 <= X[1] = X[0] * (1.5 - (V/2)*X[0]**2)
;
        MPYF    R1,R1,R2        ; R2 <= X[1]**2
        MPYF    R0,R2           ; R2 <= (V/2) * X[1]**2
        SUBRF   CNST2,R2        ; R2 <= 1.5 - (V/2) * X[1]**2
        MPYF    R2,R1           ; R1 <= X[2] = X[1] * (1.5 - (V/2)*X[1]**2)
;
        MPYF    R1,R1,R2        ; R2 <= X[2]**2
        MPYF    R0,R2           ; R2 <= (V/2) * X[2]**2
        SUBRF   CNST2,R2        ; R2 <= 1.5 - (V/2) * X[2]**2
        MPYF    R2,R1           ; R1 <= X[3] = X[2] * (1.5 - (V/2)*X[2]**2)
;
        RND     R1              ; ROUND BEFORE *
        MPYF    R1,R1,R2        ; R2 <= X[3]**2
        RND     R2              ; ROUND BEFORE *
        MPYF    R0,R2           ; R2 <= (V/2) * X[3]**2
        SUBRF   CNST2,R2        ; R2 <= 1.5 - (V/2) * X[3]**2
        RND     R2              ; ROUND BEFORE *
        MPYF    R2,R1           ; R1 <= X[4] = X[3] * (1.5 - (V/2)*X[3]**2)

;       INVERT FINAL RESULT AND RETURN

        POP     R2              ; R2 <= RETURN ADDRESS
        BUD     R2              ; RETURN (DELAYED)
        RND     R3              ; ROUND BEFORE *
        RND     R1              ; ROUND BEFORE *
        MPYF    R1,R3,R0        ; R0 = SQRT(V) = V*SQRT(1/V)





